@@ -2,5 +2,5 @@
 
 .onLoad <- function(libname, pkgname)
 {
-  library.dynam("SV", pkgname, libname)
+##  library.dynam("SV", pkgname, libname)
 }
