@@ -2,6 +2,7 @@
 #include <iostream>
 
 #include <R.h>
+//#include <Rmath.h>
 
 #include "armadillo"
 using namespace std;
@@ -20,7 +21,6 @@ using namespace arma;
 #include "ql.h"
 #include "indirect.h"
 
-
 int ParametersMulti::q;
 int ParametersMulti::p;
 
@@ -35,20 +35,13 @@ vec SetParVec(double * par, double mu, double psi, double * lambda, double * ome
 	      int nSup, int useParVec, int transf);
 vec SetParVecMulti(double * par, double * mu, double * psi, double * lambda, double * omega,
 		   double phi21, int nSup, int useParVec, int transf);
-void ConfidenceIntervals(const vec & estimate, const mat & Hi,
-			 Parameters & sd,
-			 Parameters & lower, Parameters & upper,
-			 const int transf);
-void ConfidenceIntervalsMulti(const vec & estimate, const mat & Hi,
-			      ParametersMulti & sd,
-			      ParametersMulti & lower, ParametersMulti & upper,
-			      const int transf);
-mat ComputeSandwichMatrix(mat & H, mat & gr);
+
 
 //
 extern "C" {
 
   void CheckContinuity(char ** datfile, int * nSup,
+		       int * nTimes,
 		       double * par, int * indpar,
 		       double * mu, double * psi,
 		       double * lambda, double * omega,
@@ -62,46 +55,57 @@ extern "C" {
 		       int * nEval, double * delta, double * xOut, double * xOut_transf, double * fOut,
 		       int * useRoptimiser, int * initialSteepestDescent)
   {
-    vec y0 = ReadData(*datfile);
-
-    int nTimes = y0.n_elem;
-    //vec y = y0.rows(1, nTimes-1);
-    vec y = y0;
-    nTimes = y.n_elem;
-   //  y.print("y=");
+    const int ITMAX = 200;
+    const double gradMax = 1000*1000;
+    vec y = ReadData(*datfile);
+    int ny = y.n_elem;
 
     if (*print_level >= 1) {
-      Rprintf("Number of observations: %d\n", y.n_elem);
+      Rprintf("Number of observations: %d\n", ny);
     }
-
-    if (*nObs < nTimes && *nObs > 0) {
+    // Truncate observed time series to last 'nObs' observations
+    if (*nObs < ny && *nObs > 0) {
       if (*print_level >= 1) {
 	Rprintf("Truncate series to the %d last observations ", *nObs);
       }
-      y = y.rows(nTimes-(*nObs),nTimes-1);
+      y = y.rows(ny-(*nObs),ny-1);
       if (*print_level >= 1) {
 	Rprintf("Number of observations: %d\n", y.n_elem);
       }
-      nTimes = y.n_elem;
     }
 
+    *nObs = y.n_elem - 1; // Log-return data have one less element
+
+    if (*nTimes == -1) {
+      *nTimes = *nObs;
+    }
+    else if (*nTimes < *nObs) {
+      Rprintf("Error: nObs > nTimes\nTerminating Check continuity\n");
+
+      return;
+    }
+
+    Optimise::nFuncEval = 0;
+    Optimise::nGradEval = 0;
+
     const int useStartPar = 0; // Not in use??
 
     const int nSim=1;
-    const double ftol = 0.1;
+    const double ftol = 0.001;
     const double ftol_weak = 1.0;
-    indirectExtern = new Indirect(y, nSim, *nSup, nTimes, useStartPar,
+    const int saveDraws = 0;
+    indirectExtern = new Indirect(y, nSim, *nSup, *nTimes, useStartPar,
 				  *minlambda, *maxlambda, ftol, ftol_weak, *gradtol,
-				  *transf, *addPenalty, *print_level, *useRoptimiser,
-				  *initialSteepestDescent);
+				  *transf, *addPenalty, *print_level, *useRoptimiser, gradMax,
+				  *initialSteepestDescent, ITMAX, saveDraws);
     qlExtern = indirectExtern;
 
     const vec startpar = SetParVec(par, *mu, *psi, lambda, omega, *nSup,
 				   *useParVec, *transf);
     if (*print_level >= 2) {
-      startpar.print("startpar=");
+      startpar.print_trans("startpar (unrestricted scale)=");
       Parameters par_debug(startpar, *transf);
-      par_debug.print();
+      par_debug.print("startpar (original scale)=");
     }
 
     mat funcvals;
@@ -112,6 +116,9 @@ extern "C" {
     const int npar = funcvals.n_rows;
     const int m = funcvals.n_cols;
 
+    *nFuncEval = Optimise::nFuncEval;
+    *nGradEval = Optimise::nGradEval;
+
     int index=0;
     for (int i=0;i<npar;i++) {
       for (int j=0;j<m;j++) {
@@ -120,7 +127,6 @@ extern "C" {
 	fOut[index++] = funcvals(i,j);
       }
     }
-    //    Rprintf("index= %d\n", index);
   }
 
   void SimulateVolatility(int * nSup,
@@ -145,19 +151,21 @@ extern "C" {
     const double deltaT=1.0;
     vec s2;
     Simulate sim(*nSup, *nTimes, *print_level);
-    sim.simulateInit(); // Simulates new epsilon. Sets new seed
-    const vec logYRet = sim.simulate(pex.mu, pex.lambda, pex.psi,
-				     pex.omega, *nTimes, deltaT, resetSeed, s2);
 
+    int ind=0;
+    for (int isim=0;isim<*nSim;isim++) {
+      sim.simulateInit(); // Simulates new epsilon. Sets new seed
+      const vec logYRet = sim.simulate(pex.mu, pex.lambda, pex.psi,
+				       pex.omega, *nTimes, deltaT, resetSeed, s2);
 
-
-    if (logYRet.n_elem == 0) {
-      return;
-    }
-
-    for (int i=0;i<(*nTimes);i++) {
-      s2_out[i] = s2(i);
-      logYRet_out[i] = logYRet(i);
+      if (logYRet.n_elem == 0) {
+	return;
+      }
+      
+      for (int i=0;i<(*nTimes);i++) {
+	s2_out[ind] = s2(i);
+	logYRet_out[ind++] = logYRet(i);
+      }
     }
 
   }
@@ -199,7 +207,6 @@ extern "C" {
 					  pex.omega, pex.phi(2,1), *nTimes, deltaT, resetSeed, s2);
     
 
-
     if (logYRet.n_elem == 0) {
       return;
     }
@@ -224,7 +231,8 @@ extern "C" {
 		       double * mu, double * psi,
 		       double * lambda, double * omega,
 		       double * minlambda, double * maxlambda,
-		       double * H,
+		       double * HiUnCorr,
+		       double * HiRob,
 		       int * nFuncEval, int * nGradEval,
 		       double * gradtol, int * nObs,
 		       int * transf, int * useParVec,
@@ -232,19 +240,15 @@ extern "C" {
 		       int * print_level,
 		       int * useRoptimiser,
 		       int * updatePars_,
-		       int * sandwich)
+		       int * sandwich,
+		       double * gradMax,
+		       int * nIter)
   {
-    vec y0 = ReadData(*datfile);
-
-    int n = y0.n_elem;
-    vec y = y0;
-    //  y.print("y=");
-    n = y.n_elem;
+    vec y = ReadData(*datfile);
+    const int n = y.n_elem;
 
     if (*print_level >= 1) {
       Rprintf("Number of observations: %d\n", y.n_elem);
-      //      Rprintf("Transf %d\n", *transf);
-      //      Rprintf("Useparvec %d\n", *useParVec);
     }
 
     if (*nObs < n && *nObs > 0) {
@@ -257,18 +261,19 @@ extern "C" {
       }
     }
 
-    qlExtern = new QL(y, *minlambda, *maxlambda, *transf, *addPenalty, *useRoptimiser);
+    *nObs = y.n_elem - 1; // Log-return data have one less element
+
+    qlExtern = new QL(y, *minlambda, *maxlambda, *transf, *addPenalty, *useRoptimiser, *gradMax);
 
 
     const vec parvec = SetParVec(par, *mu, *psi, lambda, omega, *nSup,
 				 *useParVec, *transf);
 
     if (*print_level >= 1) {
-      parvec.print("Start parameters=");
+      parvec.print_trans("Start parameters (unrestricted)=");
       Parameters par_debug(parvec, *transf);
-      par_debug.print();
+      par_debug.print("Start parameters (restricted)=");
     }
-    //    Rprintf("Before qlExtern->optimise\n");
 
     int npar = parvec.n_elem;
     ivec updatePars(npar);
@@ -279,8 +284,6 @@ extern "C" {
       qlExtern->checkGradient(&func, parvec, 1e-4, 1e-2, 1);
     }
 
-    //    Rprintf("QuasiLikelihood check: func %6.4f\n", func(parvec, 0).f);
-
     Optimise::nFuncEval = 0;
     Optimise::nGradEval = 0;
     for (int i=0;i<npar;i++) { // Set the parameters to estimate
@@ -291,30 +294,19 @@ extern "C" {
 					      *print_level,
 					      updatePars,
 					      func);
+    *nIter = obj.nIter;
+
+
     // Extract mu, lambda, psi, omega
     Parameters sd(*nSup);
     Parameters lower(*nSup);
     Parameters upper(*nSup);
+    Parameters lowerUn(*nSup);
+    Parameters upperUn(*nSup);
+
     mat Hi;
-    if (*sandwich) {
-      //      qlExtern->checkGradient(&func, obj.par, 1e-6, 1e-2, 1);
-
-      mat gr = qlExtern->quasiLikelihood_individual(obj.par);
-      //      vec gr_mean = mean(gr, 1);
-      //      gr_mean.print("gr_mean=");
-      //      mat gr_var = var(gr, 0, 1);
-      //      gr_var.print("gr_var=");
-
-      Hi = ComputeSandwichMatrix(obj.H, gr);
-      //      Hi.print("Hi=");
-    }
-    else {
-      Hi = inv(obj.H);
-    }
-    // Extract confidence intervals for parameters
-    ConfidenceIntervals(obj.par, Hi, sd, lower, upper, *transf);
+    qlExtern->confidenceIntervals(obj.par, obj.H, Hi, sd, lower, upper, lowerUn, upperUn, *sandwich);
     
-
     // Extract mu, lambda, psi, omega, phi21
     Parameters parObj(obj.par, *transf);
 
@@ -345,20 +337,14 @@ extern "C" {
       ind++;
     }
     ind=0;
+    const mat Hi_uncorr = inv(obj.H);
     for (int i=0;i<npar;i++) {
       for (int j=0;j<npar;j++) {
-	H[ind++] = obj.H(i,j);
+	HiUnCorr[ind] = Hi_uncorr(i,j);
+	HiRob[ind++] = Hi(i,j);
       }
     }
   
-    //    Parameters parObj(obj.par, *transf);
-    //    *mu = parObj.mu;
-    //    *psi = parObj.psi;
-    //    for (int i=0;i<(*nSup);i++) {
-    //      lambda[i] = parObj.lambda(i);
-    //      omega[i] = parObj.omega(i);
-    //    }
-    
     *nFuncEval = Optimise::nFuncEval;
     *nGradEval = Optimise::nGradEval;
 
@@ -382,36 +368,33 @@ extern "C" {
 			    int * print_level,
 			    int * useRoptimiser,
 			    int * updatePars_,
-			    int * sandwich)
+			    int * sandwich,
+			    double * gradMax,
+			    int * nIter)
   {
     mat y = ReadDataMulti(*datfile);
-
-    ParametersMulti::p = 1;
-    ParametersMulti::q = 2;
-
-    const int p_q = ParametersMulti::p + ParametersMulti::q;
-
-    int n = y.n_rows;
-    //    vec y = y0;
-    //  y.print("y=");
-    //    n = y.n_elem;
+    const int ny = y.n_rows;
 
     if (*print_level >= 1) {
-      Rprintf("Number of observations: %d\n", n);
-      //      Rprintf("Transf %d\n", *transf);
-      //      Rprintf("Useparvec %d\n", *useParVec);
+      Rprintf("Number of observations: %d\n", ny);
     }
 
-    if (*nObs < n && *nObs > 0) {
+    if (*nObs < ny && *nObs > 0) {
       if (*print_level >= 1) {
 	Rprintf("Truncate series to the %d last observations ", *nObs);
       }
-      y = y.rows(n-(*nObs),n-1);
+      y = y.rows(ny-(*nObs),ny-1);
       if (*print_level >= 1) {
 	Rprintf("Number of observations: %d\n", y.n_elem);
       }
     }
 
+    *nObs = y.n_elem - 1; // Log-return data have one less element
+
+    ParametersMulti::p = 1;
+    ParametersMulti::q = 2;
+
+    const int p_q = ParametersMulti::p + ParametersMulti::q;
     
     vec minlambdavec(p_q);
     vec maxlambdavec(p_q);
@@ -419,17 +402,16 @@ extern "C" {
       minlambdavec(k) = minlambda[k];
       maxlambdavec(k) = maxlambda[k];
     }
-    qlExtern = new QL(y, minlambdavec, maxlambdavec, *transf, *addPenalty, *useRoptimiser);
+    qlExtern = new QL(y, minlambdavec, maxlambdavec, *transf, *addPenalty, *useRoptimiser, *gradMax);
 
 
     const vec parvec = SetParVecMulti(par, mu, psi, lambda, omega, *phi21,
 				      *nSup, *useParVec, *transf);
 
     if (*print_level >= 1) {
-      cout << "Start parameters (transformed): " << trans(parvec);
+      parvec.print_trans("Start parameters (unrestricted):");
       ParametersMulti par_debug(parvec, *transf);
-      cout << "Start parameters (original scale): " << endl;
-      par_debug.print();
+      par_debug.print("Start parameters (restricted):");
     }
 
     int npar = parvec.n_elem;
@@ -457,31 +439,17 @@ extern "C" {
 					      *print_level,
 					      updatePars,
 					      funcMulti);
+    *nIter = obj.nIter;
 
     ParametersMulti sd(*nSup);
     ParametersMulti lower(*nSup);
     ParametersMulti upper(*nSup);
-    mat Hi;
-     if (*sandwich) {
-       //      qlExtern->checkGradient(&funcMulti, obj.par, 1e-6, 1e-2, 1);
 
-      mat gr = qlExtern->quasiLikelihoodMulti_individual(obj.par);
-      //      vec gr_mean = mean(gr, 1);
-      //      gr_mean.print("gr_mean=");
-      //      mat gr_var = var(gr, 0, 1);
-      //      gr_var.print("gr_var=");
-
-      Hi = ComputeSandwichMatrix(obj.H, gr);
-      //      Hi.print("Hi=");
-    }
-    else {
-      Hi = inv(obj.H);
-    }
      // Extract confidence intervals for parameters
-    ConfidenceIntervalsMulti(obj.par, Hi, sd, lower, upper, *transf);
-
-    // Extract confidence intervals for parameters
-    
+    ParametersMulti lowerUn(*nSup);
+    ParametersMulti upperUn(*nSup);
+    mat Hi;
+    qlExtern->confidenceIntervalsMulti(obj.par, obj.H, Hi, sd, lower, upper, lowerUn, upperUn, *sandwich);
 
     // Extract mu, lambda, psi, omega, phi21
     ParametersMulti parObj(obj.par, *transf);
@@ -535,6 +503,7 @@ extern "C" {
   }
 
   void IndirectInference(char ** datfile, int * nSup, int * nSim,
+			 int * nTimes,
 			 double * par,
 			 double * mu, double * psi,
 			 double * lambda, double * omega,
@@ -551,69 +520,154 @@ extern "C" {
 			 int * print_level,
 			 int * useRoptimiser,
 			 int * initialSteepestDescent,
-			 int * nSimAll)
+			 int * ITMAX,
+			 int * nSimAll,
+			 double * funcval,
+			 int * niter,
+			 int * convergence,
+			 char ** simfile,
+			 int * error,
+			 int * useQLAsStartPar,
+			 double * gradMax)
   {
-    vec y0 = ReadData(*datfile);
-
-    int nTimes = y0.n_elem;
-    vec y = y0;//.rows(1, nTimes-1);
-    //  y.print("y=");
+    vec y = ReadData(*datfile);
+    const int ny = y.n_elem;
 
     if (*print_level >= 1) {
       Rprintf("Number of observations: %d\n", y.n_elem);
     }
-    if (*nObs < nTimes && *nObs > 0) {
+    // Truncate observed time series to last 'nObs' observations
+    if (*nObs < ny && *nObs > 0) {
       if (*print_level >= 1) {
 	Rprintf("Truncate series to the %d last observations ", *nObs);
       }
-      y = y.rows(nTimes-(*nObs),nTimes-1);
+      y = y.rows(ny-(*nObs),ny-1);
       if (*print_level >= 1) {
 	Rprintf("Number of observations: %d\n", y.n_elem);
       }
     }
 
-    const int useStartPar = 0; // Not in use??
+    *nObs = y.n_elem - 1; // Log-return data have one less element
 
-    indirectExtern = new Indirect(y, *nSim, *nSup, nTimes, useStartPar,
+    if (*nTimes == -1) {
+      *nTimes = *nObs;
+    }
+    else if (*nTimes < *nObs) {
+      Rprintf("Error: nObs > nTimes\nTerminating Indirect inference\n");
+
+      return;
+    }
+    
+
+    int sandwichIndirect;
+    if (*nSim > 1)
+      sandwichIndirect = 0;
+    else
+      sandwichIndirect = 1;
+
+
+    const int useStartPar = 0; // Not in use??
+    const int saveDraws = 0;
+    indirectExtern = new Indirect(y, *nSim, *nSup, *nTimes, useStartPar,
 				  *minlambda, *maxlambda, *ftol, *ftol_weak, *gradtol,
 				  *transf, *addPenalty, *print_level,
-				  *useRoptimiser, *initialSteepestDescent);
+				  *useRoptimiser, *gradMax, *initialSteepestDescent, *ITMAX, saveDraws);
     qlExtern = indirectExtern;
 
     const vec startpar = SetParVec(par, *mu, *psi, lambda, omega, *nSup,
 				   *useParVec, *transf);
-   if (*print_level >= 2) {
-     startpar.print("startpar=");
-     Parameters par_debug(startpar, *transf);
-     par_debug.print();
-   }
+    if (*print_level >= 2) {
+      startpar.print_trans("Start parameters (unrestricted)=");
+      Parameters par_debug(startpar, *transf);
+      par_debug.print("Start parameters (restricted)=");
+    }
 
     Optimise::nFuncEval = 0;
     Optimise::nGradEval = 0;
     Optimise::nFuncEvalOuter = 0;
 
-    EstimationObject obj = indirectExtern->indirectInference(startpar);
+    EstimationObject obj = indirectExtern->indirectInference(startpar, *useQLAsStartPar);
 
     if (obj.status == EXIT_FAILURE) {
       if (*print_level >= 1) {
 	Rprintf("Indirect inference aborted\n");
       }
+      *error = 1;
       return;
     }
 
-    // Extract mu, lambda, psi, omega
+    Parameters sd(*nSup);
+    Parameters lower(*nSup);
+    Parameters upper(*nSup);
+    Parameters lowerUn(*nSup);
+    Parameters upperUn(*nSup);
+    Parameters sd2(*nSup);
+    Parameters lower2(*nSup);
+    Parameters upper2(*nSup);
+
+    indirectExtern->confidenceIntervalsIndirect(obj, sd, lower, upper, lowerUn, upperUn, sandwichIndirect);
+
     Parameters parObj(obj.par, *transf);
-    *mu = parObj.mu;
-    *psi = parObj.psi;
+
+    if (strcmp(*simfile, "") != 0) { // Write to simfile
+      if (*print_level >= 1) {
+	Rprintf("Writes to file simulated data corresponding to the indirect inference estimate\n");
+      }
+      vec ysim = indirectExtern->simulateData(parObj);
+      writeData(ysim, *simfile);
+    }
+
+    // Extract mu, lambda, psi, omega
+    mu[0] = parObj.mu;
+    mu[1] = sd.mu;
+    mu[2] = lower.mu;
+    mu[3] = upper.mu;
+    if (sandwichIndirect && 0) {
+      mu[4] = sd2.mu;
+      mu[5] = lower2.mu;
+      mu[6] = upper2.mu;
+    }
+
+    psi[0] = parObj.psi;
+    psi[1] = sd.psi;
+    psi[2] = lower.psi;
+    psi[3] = upper.psi;
+    if (sandwichIndirect && 0) {
+      psi[4] = sd2.psi;
+      psi[5] = lower2.psi;
+      psi[6] = upper2.psi;
+    }
+
+    int ind=0;
+
     for (int i=0;i<(*nSup);i++) {
-      lambda[i] = parObj.lambda(i);
-      omega[i] = parObj.omega(i);
+      lambda[ind] = parObj.lambda(i);
+      lambda[(*nSup)+ind] = sd.lambda(i);
+      lambda[2*(*nSup)+ind] = lower.lambda(i);
+      lambda[3*(*nSup)+ind] = upper.lambda(i);
+      if (sandwichIndirect && 0) {
+	lambda[4*(*nSup)+ind] = sd2.lambda(i);
+	lambda[5*(*nSup)+ind] = lower2.lambda(i);
+	lambda[6*(*nSup)+ind] = upper2.lambda(i);
+      }
+
+      omega[ind] = parObj.omega(i);
+      omega[(*nSup)+ind] = sd.omega(i);
+      omega[2*(*nSup)+ind] = lower.omega(i);
+      omega[3*(*nSup)+ind] = upper.omega(i);
+      if (sandwichIndirect && 0) {
+	omega[4*(*nSup)+ind] = sd.omega(i);
+	omega[5*(*nSup)+ind] = lower.omega(i);
+	omega[6*(*nSup)+ind] = upper.omega(i);
+      }
+
+      ind++;
     }
 
     *nSimAll = obj.nSimAll;
 
     int k=0;
-    for (int i=0;i<(*nSimAll);i++) {
+    for (int i=0;i<(*nSim);i++) {
       Parameters parSim(obj.parsim.col(i), *transf);
       muSim[i] = parSim.mu;
       psiSim[i] = parSim.psi;
@@ -624,6 +678,17 @@ extern "C" {
       }
     }
 
+    for (int i=0;i<(*nSimAll);i++) {
+      funcval[i] = obj.funcvalAll(i);
+      niter[i] = obj.iterAll(i);
+      if (obj.funcvalAll(i) > (*ftol_weak))
+	convergence[i] = 0;
+      else if (obj.funcvalAll(i) > (*ftol))
+	convergence[i] = 1;
+      else
+	convergence[i] = 2;
+    }
+
     *nFuncEval = Optimise::nFuncEval;
     *nGradEval = Optimise::nGradEval;
     *nFuncEvalOuter = Optimise::nFuncEvalOuter;
@@ -633,11 +698,17 @@ extern "C" {
     }
   }
 
+  //  Simulation study for univariate data
   void SimulationStudy(int * nRep, int * methods,
 		       int * nSup, int * nSimIndirect,
-		       int * nTimes,
+		       int * nObs,
+		       int * nTimes, double * par,
 		       double * mu, double * psi,
 		       double * lambda, double * omega,
+		       int * covmu, int * covpsi,
+		       int * covlambda, int * covomega,
+		       double * funcvals, int * iters,
+		       int * nsimIndTot,
 		       double * minlambda, double * maxlambda,
 		       int * nFuncEval, int * nGradEval,
 		       int * nFuncEvalOuter,
@@ -648,32 +719,259 @@ extern "C" {
 		       int * print_level,
 		       int * useRoptimiser,
 		       int * initialSteepestDescent,
-		       int * error)
+		       int * ITMAX,
+		       int * error,
+		       char ** savefile, char ** savefile2,
+		       int * sandwich,
+		       int * sandwichIndirect,
+		       int * writeSimDataToFile,
+		       double * gradMax)
   {
-    Rprintf("SimulationStudy not yet implemented\n");
- //    for (int i=0; i<(*nRep); i++) {
+    Simulate simObs(*nSup, *nObs, *print_level);
+    const int useParVec = 0;
+    const int resetSeed = 0;
+    const double deltaT = 1.0;
+    const int useStartPar = 0; // Not in use??
+
+    vec y = zeros<vec>(1); // dummy
+    const int saveDraws = 0;
+    indirectExtern = new Indirect(y, *nSimIndirect, *nSup, *nTimes, useStartPar,
+				  *minlambda, *maxlambda, *ftol, *ftol_weak, *gradtol,
+				  *transf, *addPenalty, *print_level,
+				  *useRoptimiser, *gradMax, *initialSteepestDescent, *ITMAX, saveDraws);
+    qlExtern = indirectExtern;
+    const vec parvec = SetParVec(par, *mu, *psi, lambda, omega, *nSup,
+				 useParVec, *transf);
+    Parameters pex(parvec, *transf);
+    Parameters truepar(parvec, NOTRANSF);
+    
+    ivec updatePars = ones<ivec>(parvec.n_elem);
+    const int npar = parvec.n_elem;
+
+    const int iQL = 0;
+    const int iII = 1;
+    int nMethods=0;
+    const int nMaxMethods=2;
+    for (int i=0;i<nMaxMethods;i++) {
+      if (methods[i] == 1)
+	nMethods++;
+    }
+
+    int nExtra = 0;
+    if (methods[iII]) {
+      nExtra = 0; // 1;
+    }
+
+    mat mumat(*nRep, 2*nMethods+nExtra);
+    mat psimat(*nRep, 2*nMethods+nExtra);
+    mat lambdamat(*nRep, (2*nMethods+nExtra)*(*nSup));
+    mat omegamat(*nRep, (2*nMethods+nExtra)*(*nSup));
+    imat coverageMu(*nRep, nMethods+nExtra);
+    imat coveragePsi(*nRep, nMethods+nExtra);
+    imat coverageLambda(*nRep, (nMethods+nExtra)*(*nSup));
+    imat coverageOmega(*nRep, (nMethods+nExtra)*(*nSup));
+    imat itervec(*nRep, *nSimIndirect);
+    mat funcvalvec(*nRep, *nSimIndirect);
+    ivec nSimAll(*nRep);
+
+    ofstream ost(*savefile);
+    ost.precision(8);
+    ofstream ost2(*savefile2);
+    ost2.precision(8);
+
+    for (int i=0; i<(*nRep); i++) {
+      Rprintf("Replicate %d ========\n\n", i);
       // Simulate data, write to file
+      simObs.simulateInit(); // Simulates new epsilon. Sets new seed
+      vec s2;
+      // Simulates nObs log-returns observations
+      vec ysim = simObs.simulate(pex.mu, pex.lambda, pex.psi,
+				 pex.omega, *nObs, deltaT, resetSeed, s2);
+      if (*writeSimDataToFile) {
+	char nc[20];
+	sprintf(nc, "%d", i);
+	char filename[80];
+	strcpy(filename,"sim_tmp");
+	strcat(filename, nc);
+	strcat(filename,".dat");
 
-      // QL + Indirect inference
+	writeData(ysim, filename);
+      }
+      qlExtern->setDataLogReturns(ysim);
+      qlExtern->setNObs(ysim.n_elem);
+
+      simObs.cleanup();
+
+      Parameters sd(*nSup);
+      Parameters lowerUn(*nSup);
+      Parameters upperUn(*nSup);
+      Parameters lower(*nSup);
+      Parameters upper(*nSup);
+
+      int ind1=0;
+      int ind2=0;
+      int ind3=0;
+      int ind4=0;
+
+      EstimationObject objQL;
+      EstimationObject objII;
+      mat HiQL;
+      mat grQL;
+      vec sdQL_transf;
+       if (methods[iQL]) {
+	// QL
+	objQL = qlExtern->optimise(parvec,
+				   *gradtol,
+				   *print_level,
+				   updatePars,
+				   func);
+      }
+      if (methods[iII]) {
+	objII = indirectExtern->indirectInference(parvec);
+	if (objII.status == EXIT_FAILURE) {
+	  Rprintf("Try a new replicate %d\n", i);
+	  i--;
+	  continue;
+	}
+       }
+
+      if (methods[iQL]) {
+	Parameters pexQL(objQL.par, *transf);
+	pexQL.print("QL estimated");
+	qlExtern->confidenceIntervals(objQL.par, objQL.H, HiQL, sd, lower, upper, lowerUn, upperUn, *sandwich);
+
+	// Estimates
+	mumat(i,ind1) = pexQL.mu;
+	psimat(i,ind1++) = pexQL.psi;
+	lambdamat.submat(i,0, i, *nSup-1) = trans(pexQL.lambda);
+	omegamat.submat(i,0, i, *nSup-1) = trans(pexQL.omega);
+	ind2 += *nSup;
+
+	// SD
+	mumat(i,ind1) = sd.mu;
+	psimat(i,ind1++) = sd.psi;
+	lambdamat.submat(i,ind2, i, ind2+(*nSup)-1) = trans(sd.lambda);
+	omegamat.submat(i,ind2, i, ind2+(*nSup)-1) = trans(sd.omega);
+	ind2 += *nSup;
+     
+	// Coverage
+	coverageMu(i,ind3) = (truepar.mu >= lowerUn.mu) && (truepar.mu <= upperUn.mu);
+	coveragePsi(i,ind3++) = (truepar.psi >= lowerUn.psi) && (truepar.psi <= upperUn.psi);
+
+	for (int j=0;j<*nSup;j++) {
+	  coverageLambda(i,j) = (truepar.lambda(j) >= lowerUn.lambda(j)) && (truepar.lambda(j) <= upperUn.lambda(j));
+	  coverageOmega(i,j) = (truepar.omega(j) >= lowerUn.omega(j)) && (truepar.omega(j) <= upperUn.omega(j));
+	}
+	ind4 += *nSup;
+      }
+
+      if (methods[iII]) {
+	indirectExtern->confidenceIntervalsIndirect(objII, sd, lower, upper, lowerUn, upperUn, *sandwichIndirect);
+
+	Parameters pexII(objII.par, *transf);
+	pexII.print("II estimate");
+
+	// Estimates
+	mumat(i,ind1) = pexII.mu;
+	psimat(i,ind1++) = pexII.psi;
+	lambdamat.submat(i,ind2, i, ind2+(*nSup)-1) = trans(pexII.lambda);
+	omegamat.submat(i,ind2, i, ind2+(*nSup)-1) = trans(pexII.omega);
+	ind2 += *nSup;
+
+	// SD
+	mumat(i,ind1) = sd.mu;
+	psimat(i,ind1++) = sd.psi;
+	lambdamat.submat(i,ind2, i, ind2+(*nSup)-1) = trans(sd.lambda);
+	omegamat.submat(i,ind2, i, ind2+(*nSup)-1) = trans(sd.omega);
+	ind2 += *nSup;
 
-      // Keep estimates
-  //      muvec(i) = 0.0;
-  //      psivec(i) = 0.0;
-  //     lambdamat.row(i) = 0.0;
-  //     omegamat.row(i) = 0.0;
-  //     funcval(i) = 0.0;
-  //   }
   
+	// Coverage
+	coverageMu(i,ind3) = (truepar.mu >= lowerUn.mu) && (truepar.mu <= upperUn.mu);
+	coveragePsi(i,ind3++) = (truepar.psi >= lowerUn.psi) && (truepar.psi <= upperUn.psi);
+
+	for (int j=0;j<*nSup;j++) {
+	  coverageLambda(i,ind4+j) = (truepar.lambda(j) >= lowerUn.lambda(j)) && (truepar.lambda(j) <= upperUn.lambda(j));
+	  coverageOmega(i,ind4+j) = (truepar.omega(j) >= lowerUn.omega(j)) && (truepar.omega(j) <= upperUn.omega(j));
+	}
+	ind4 += *nSup;
+ 
+	funcvalvec.row(i) = trans(objII.funcval);
+	nSimAll(i) = objII.nSimAll;
+
+
+	ost2 << i << " ";
+	ost2 << nSimAll(i)  << " ";
+	for (int j=0;j<nSimAll(i);j++) ost2 << objII.funcvalAll(j) << " ";
+	for (int j=0;j<nSimAll(i);j++) ost2 << objII.iterAll(j) << " ";
+	ost2 << endl;
+      }
+      // Save to file
+      ost << i << " ";
+      for (int j=0;j<2*nMethods;j++) ost << mumat(i, j) << " ";
+      for (int j=0;j<2*nMethods;j++) ost << psimat(i, j) << " ";
+      for (int j=0;j<2*nMethods*(*nSup);j++) ost << lambdamat(i, j) << " ";
+      for (int j=0;j<2*nMethods*(*nSup);j++) ost << omegamat(i, j) << " ";
+      for (int j=0;j<nMethods;j++) ost << coverageMu(i,j) << " ";
+      for (int j=0;j<nMethods;j++) ost << coveragePsi(i,j) << " ";
+      for (int j=0;j<nMethods*(*nSup);j++) ost << coverageLambda(i,j) << " ";
+      for (int j=0;j<nMethods*(*nSup);j++) ost << coverageOmega(i,j) << " ";
+      for (int j=0;j<*nSimIndirect;j++) ost << funcvalvec(i,j) << " ";
+      for (int j=0;j<*nSimIndirect;j++) ost << itervec(i,j) << " ";
+      //      for (int j=0;j<*nSimIndirect;j++) ost << niter(i,j) << " ";
+      ost << nSimAll(i) << " ";
+      if (methods[iQL]) {
+	const mat Hi_uncorr = inv(objQL.H);
+	for (int j=0;j<npar;j++) ost << sqrt(Hi_uncorr(j,j)) << " ";
+	for (int j=0;j<npar;j++) ost << sqrt(HiQL(j,j)) << " ";
+      }
+      if (methods[iII]) {
+	for (int j=0;j<*nSimIndirect;j++) for (int k=0;k<npar;k++) ost << objII.parsim(k,j) << " ";
+      }
+      ost << endl;
+    }
+    ost.close();
+    ost2.close();
+
+    int ind1=0;
+    int ind2=0;
+    int ind3=0;
+    int ind4=0;
+    int ind5=0;
+    for (int i=0;i<*nRep;i++) {
+      for (int j=0;j<2*nMethods;j++) {
+	mu[ind1] = mumat(i,j);
+	psi[ind1++] = psimat(i,j);
+      }
+      for (int j=0;j<nMethods;j++) {
+	covmu[ind2] = coverageMu(i,j);
+	covpsi[ind2++] = coveragePsi(i,j);
+      }
+      for (int j=0;j<2*nMethods*(*nSup);j++) {
+	lambda[ind3] = lambdamat(i,j);
+	omega[ind3++] = omegamat(i,j);
+      }
+      for (int j=0;j<nMethods*(*nSup);j++) {
+	covlambda[ind4] = coverageLambda(i,j);
+	covomega[ind4++] = coverageOmega(i,j);
+      }
+      for (int j=0;j<*nSimIndirect;j++) {
+	funcvals[ind5] = funcvalvec(i,j);
+	iters[ind5++] = itervec(i,j);
+      }
+      nsimIndTot[i] = nSimAll(i);
+    }
   }
 
 
   void test(double * par, int * npar,
-	   int * nFuncEvalOuter,
-	   int * print_level,
-	   int * initialSteepestDescent)
+	    int * nFuncEvalOuter,
+	    int * print_level,
+	    int * initialSteepestDescent)
   {
 
     const int useStartPar = 0; // Not in use??
+    const double gradMax = 1000*1000;
     const int useRoptimiser = 0;
     const double minlambda = 0.1;
     const double maxlambda = 1.0;
@@ -681,6 +979,7 @@ extern "C" {
     const double gradtol = 1e-4;
     const double ftol = 1e-4;
     const double ftol_weak = 1e-3;
+    const int ITMAX = 200;
     const int nSim = 1;
     const int nSup = 1;
     const int addPenalty = 0;
@@ -688,10 +987,11 @@ extern "C" {
     vec y = zeros<vec>(2);
     // nTimes --> nTimes - 1 ???????
     //    qlExtern = new QL(y, *minlambda, *maxlambda, *transf, *addPenalty);
+    const int saveDraws = 0;
     indirectExtern = new Indirect(y, nSim, nSup, nTimes, useStartPar,
 				  minlambda, maxlambda, ftol, ftol_weak, gradtol,
 				  transf, addPenalty, *print_level,
-				  useRoptimiser, *initialSteepestDescent);
+				  useRoptimiser, gradMax, *initialSteepestDescent, ITMAX, saveDraws);
     qlExtern = indirectExtern;
 
     vec startpar(*npar);
@@ -699,7 +999,7 @@ extern "C" {
       startpar(i) = par[i];
     }
     if (*print_level >= 2) {
-      startpar.print("startpar=");
+      startpar.print_trans("startpar=");
     }
 
     Optimise::nFuncEvalOuter = 0;
@@ -734,23 +1034,19 @@ vec ReadData(string filename) {
 
   vec y(nSize);
   double y0;
-  //  ist >> y0;
   while(ist >> y0) {
     y(index++) = y0;
-    //    if (index < 10)
-    //      cout << y(index-1) << endl;
+
     if (index >= nSize) {
       nSize += nDeltaSize;
       vec ycopy = y.rows(0, index-1);
       y.set_size(nSize);
       y.rows(0, index-1) = ycopy;
     }
-	
-    //    ist >> y0;
   }
   if (0) {
     vec ysub = y.rows(0,10);
-    ysub.print("y=");
+    ysub.print_trans("y=");
   }
 
   return y.rows(0, index-1);
@@ -800,7 +1096,7 @@ mat ReadDataMulti(string filename) {
 
   if (0) {
     mat ysub = y.rows(0,10);
-    ysub.print("y=");
+    ysub.print_trans("y=");
     Rprintf("Number of lines removed: %d\n", nZeroLines);
   }
 
@@ -826,9 +1122,6 @@ vec SetParVec(double * par, double mu, double psi, double * lambda, double * ome
     }
     Parameters par(mu, psi, omegavec, lambdavec, transf);
     parvec = par.extractParsInv(transf);
-    
-    // Print parameters
-    //    par.print();
   }
 
   return parvec;
@@ -869,110 +1162,3 @@ vec SetParVecMulti(double * par, double * mu, double * psi, double * lambda, dou
 
   return parvec;
 }
-
-void ConfidenceIntervals(const vec & estimate, const mat & Hi,
-			 Parameters & sd,
-			 Parameters & lower, Parameters & upper,
-			 const int transf) {
-  const double q0_975 = 1.959964;
-  //  mat Hi = inv(H);
-  const vec sd_transf = sqrt(Hi.diag());
-  vec lowervec = estimate - q0_975*sd_transf;
-  vec uppervec = estimate + q0_975*sd_transf;
-
-  lower.setPars(lowervec, transf);
-  upper.setPars(uppervec, transf);
-
-  const int npar = estimate.n_elem;
-  const int nsup = Parameters::numberOfSuperPositions(estimate);
-  Parameters x(nsup);
-  const int nsim=10000;
-  vec sumx = zeros<vec>(npar);
-  vec sumx2 = zeros<vec>(npar);
-  GetRNGstate();
-  for (int i=0;i<nsim;i++) {
-    // Draw random normal numbers with mean=estimate, sd=sd_transf
-    const vec eps = Simulate::normal(npar);
-    const vec u = estimate + eps % sd_transf;
-
-    // Transform u
-    x.setPars(u, transf);
-    const vec xvec = x.asvector();
-    sumx = sumx + xvec;
-    sumx2 = sumx2 + xvec%xvec;
-  }
-  PutRNGstate();
-  vec varvec = (sumx2 - (1.0/nsim)*sumx%sumx)/(nsim-1.0);
-  //  varvec.print("sdvec=");
-  vec sdvec = sqrt(varvec);
-  //  sdvec.print("sdvec=");
-  sd.setPars(sdvec, NOTRANSF);
-  //  sd.print();
-}
-
-void ConfidenceIntervalsMulti(const vec & estimate, const mat & Hi,
-			      ParametersMulti & sd,
-			      ParametersMulti & lower, ParametersMulti & upper,
-			      const int transf) {
-  const double q0_975 = 1.959964;
-  //  mat Hi = inv(H);
-  const vec sd_transf = sqrt(Hi.diag());
-  vec lowervec = estimate - q0_975*sd_transf;
-  vec uppervec = estimate + q0_975*sd_transf;
-
-  const int npar = estimate.n_elem;
-  const int nsup = ParametersMulti::numberOfSuperPositions(estimate);
-  ParametersMulti x(nsup);
-  const int nsim=1000;
-  vec sumx = zeros<vec>(npar);
-  vec sumx2 = zeros<vec>(npar);
-  GetRNGstate();
-  for (int i=0;i<nsim;i++) {
-    // Draw random normal numbers with mean=estimate, sd=sd_transf
-    const vec eps = Simulate::normal(npar);
-    const vec u = estimate + eps % sd_transf;
-
-    // Transform u
-    x.setPars(u, transf);
-    const vec xvec = x.asvector();
-    sumx = sumx + xvec;
-    sumx2 = sumx2 + xvec%xvec;
-  }
-  PutRNGstate();
-  vec varvec = (sumx2 - (1.0/nsim)*sumx%sumx)/(nsim-1.0);
-  //  varvec.print("sdvec=");
-  vec sdvec = sqrt(varvec);
-  //  sdvec.print("sdvec=");
-  sd.setPars(sdvec, NOTRANSF);
-  //  sd.print();
-
-  lower.setPars(lowervec, transf);
-  upper.setPars(uppervec, transf);
-}
-
-mat ComputeSandwichMatrix(mat & H, mat & gr) {
-  const double gradMax = 1000.0;
-  rowvec grt = sum(gr % gr,0);
-  int ap = gr.n_cols;
-  int ind=0;
-  for (int t=0;t<ap;t++) {
-    if (grt(t) < gradMax) {
-      gr.col(ind++) = gr.col(t);
-    }
-    //    else {
-    //      Rprintf("grt(%d)=%6.4f\n", t, grt(t));
-    //    }
-  }
-
-  //  Rprintf("ind=%d, ap=%d\n", ind, ap);
-  gr = gr.cols(0, ind-1);
-  double mult = ((double) ap)/((double) ind);
-  gr = mult * gr;
-
-  mat I = gr * trans(gr);
-  mat Hi = inv(H);
-  
-  mat S = Hi*I*Hi;
-
-  return S;
-}
